@@ -1,13 +1,18 @@
 import asyncio
 import socket
-import sys
 from types import ModuleType
-from typing import Tuple
+from typing import List, Optional, Sequence, Set, Tuple, Union
 from unittest import mock
 
 import pytest
 
-from aiohappyeyeballs import start_connection
+from aiohappyeyeballs import (
+    AddrInfoType,
+    SocketFactoryType,
+    _staggered,
+    impl,
+    start_connection,
+)
 
 
 def mock_socket_module():
@@ -137,6 +142,33 @@ async def test_single_addr_success_passing_loop(m_socket: ModuleType) -> None:
         )
 
 
+@pytest.mark.asyncio
+@patch_socket
+async def test_single_addr_socket_factory(m_socket: ModuleType) -> None:
+    mock_socket = mock.MagicMock(
+        family=socket.AF_INET,
+        type=socket.SOCK_STREAM,
+        proto=socket.IPPROTO_TCP,
+        fileno=mock.MagicMock(return_value=1),
+    )
+
+    def factory(addr_info: AddrInfoType) -> socket.socket:
+        return mock_socket
+
+    addr_info = [
+        (
+            socket.AF_INET,
+            socket.SOCK_STREAM,
+            socket.IPPROTO_TCP,
+            "",
+            ("107.6.106.82", 80),
+        )
+    ]
+    loop = asyncio.get_running_loop()
+    with mock.patch.object(loop, "sock_connect", return_value=None):
+        assert await start_connection(addr_info, socket_factory=factory) == mock_socket
+
+
 @pytest.mark.asyncio
 @patch_socket
 async def test_multiple_addr_success_second_one(
@@ -180,6 +212,76 @@ async def test_multiple_addr_success_second_one(
         assert await start_connection(addr_info) == mock_socket
 
 
+@pytest.mark.asyncio
+@patch_socket
+async def test_multiple_winners_cleaned_up(
+    m_socket: ModuleType,
+) -> None:
+    loop = asyncio.get_running_loop()
+    finish = loop.create_future()
+
+    def _socket(*args, **kw):
+        return mock.MagicMock(
+            family=socket.AF_INET,
+            type=socket.SOCK_STREAM,
+            proto=socket.IPPROTO_TCP,
+            fileno=mock.MagicMock(return_value=1),
+        )
+
+    async def _connect_sock(
+        loop: asyncio.AbstractEventLoop,
+        exceptions: List[List[Union[OSError, RuntimeError]]],
+        addr_info: AddrInfoType,
+        local_addr_infos: Optional[Sequence[AddrInfoType]] = None,
+        sockets: Optional[Set[socket.socket]] = None,
+        socket_factory: Optional[SocketFactoryType] = None,
+    ) -> socket.socket:
+        await finish
+        sock = _socket()
+        assert sockets is not None
+        sockets.add(sock)
+        return sock
+
+    m_socket.socket = _socket  # type: ignore
+    addr_info = [
+        (
+            socket.AF_INET,
+            socket.SOCK_STREAM,
+            socket.IPPROTO_TCP,
+            "",
+            ("107.6.106.82", 80),
+        ),
+        (
+            socket.AF_INET,
+            socket.SOCK_STREAM,
+            socket.IPPROTO_TCP,
+            "",
+            ("107.6.106.83", 80),
+        ),
+        (
+            socket.AF_INET,
+            socket.SOCK_STREAM,
+            socket.IPPROTO_TCP,
+            "",
+            ("107.6.106.84", 80),
+        ),
+        (
+            socket.AF_INET,
+            socket.SOCK_STREAM,
+            socket.IPPROTO_TCP,
+            "",
+            ("107.6.106.85", 80),
+        ),
+    ]
+    with mock.patch.object(impl, "_connect_sock", _connect_sock):
+        task = loop.create_task(
+            start_connection(addr_info, happy_eyeballs_delay=0.0001, interleave=0)
+        )
+        await asyncio.sleep(0.1)
+        loop.call_soon(finish.set_result, None)
+        await task
+
+
 @pytest.mark.asyncio
 @patch_socket
 async def test_multiple_addr_success_second_one_happy_eyeballs(
@@ -225,6 +327,54 @@ async def test_multiple_addr_success_second_one_happy_eyeballs(
         )
 
 
+@pytest.mark.asyncio
+@patch_socket
+async def test_happy_eyeballs_socket_factory(
+    m_socket: ModuleType,
+) -> None:
+    mock_socket = mock.MagicMock(
+        family=socket.AF_INET,
+        type=socket.SOCK_STREAM,
+        proto=socket.IPPROTO_TCP,
+        fileno=mock.MagicMock(return_value=1),
+    )
+
+    idx = -1
+    errors = ["err1", "err2"]
+
+    def factory(addr_info: AddrInfoType) -> socket.socket:
+        nonlocal idx, errors
+        idx += 1
+        if idx == 1:
+            raise OSError(5, errors[idx])
+        return mock_socket
+
+    addr_info = [
+        (
+            socket.AF_INET,
+            socket.SOCK_STREAM,
+            socket.IPPROTO_TCP,
+            "",
+            ("107.6.106.82", 80),
+        ),
+        (
+            socket.AF_INET,
+            socket.SOCK_STREAM,
+            socket.IPPROTO_TCP,
+            "",
+            ("107.6.106.83", 80),
+        ),
+    ]
+    loop = asyncio.get_running_loop()
+    with mock.patch.object(loop, "sock_connect", return_value=None):
+        assert (
+            await start_connection(
+                addr_info, happy_eyeballs_delay=0.3, socket_factory=factory
+            )
+            == mock_socket
+        )
+
+
 @pytest.mark.asyncio
 @patch_socket
 async def test_multiple_addr_all_fail_happy_eyeballs(
@@ -1653,11 +1803,14 @@ async def test_uvloop_mixing_os_and_runtime_error(
 
 @patch_socket
 @pytest.mark.asyncio
-@pytest.mark.xfail(reason="raises RuntimeError: coroutine ignored GeneratorExit")
 async def test_handling_system_exit(
     m_socket: ModuleType,
 ) -> None:
     """Test handling SystemExit."""
+
+    class MockSystemExit(BaseException):
+        """Mock SystemExit."""
+
     mock_socket = mock.MagicMock(
         family=socket.AF_INET,
         type=socket.SOCK_STREAM,
@@ -1675,7 +1828,7 @@ async def test_handling_system_exit(
         sock: socket.socket, address: Tuple[str, int, int, int]
     ) -> None:
         create_calls.append(address)
-        raise SystemExit
+        raise MockSystemExit
 
     m_socket.socket = _socket  # type: ignore
     ipv6_addr_info = (
@@ -1717,9 +1870,9 @@ async def test_handling_system_exit(
         ),
     ]
     loop = asyncio.get_running_loop()
-    with pytest.raises(SystemExit), mock.patch.object(
+    with pytest.raises(MockSystemExit), mock.patch.object(
         loop, "sock_connect", _sock_connect
-    ):
+    ), mock.patch.object(_staggered, "RE_RAISE_EXCEPTIONS", (MockSystemExit,)):
         await start_connection(
             addr_info,
             happy_eyeballs_delay=0.3,
@@ -1820,13 +1973,6 @@ async def test_cancellation_is_not_swallowed(
     ]
 
 
-@pytest.mark.asyncio
-@pytest.mark.skipif(sys.version_info >= (3, 8, 2), reason="requires < python 3.8.2")
-def test_python_38_compat() -> None:
-    """Verify python < 3.8.2 compatibility."""
-    assert asyncio.futures.TimeoutError is asyncio.TimeoutError  # type: ignore[attr-defined]
-
-
 @pytest.mark.asyncio
 @pytest.mark.parametrize(
     "connect_side_effect",
